import React, { useState, useEffect } from 'react';
import {
  Tree,
  Table,
  Space,
  message,
  Modal,
  Form,
  Input,
  notification,
  Tooltip,
} from 'antd';
import {
  FileAddTwoTone,
  FolderAddTwoTone,
  EditTwoTone,
  DeleteTwoTone,
  PlayCircleOutlined,
  ShareAltOutlined,
  KeyOutlined,
  ApartmentOutlined,
} from '@ant-design/icons';
import { PageContainer } from '@ant-design/pro-layout';
// import classnames from 'classnames';
import { get } from '../../services';
import styles from './UserManage.less';

const UserManage = () => {
  const time = new Date();
  const [treeLoading, setTreeLoading] = useState(false);
  const [tableLoading, setTableLoading] = useState(false);
  const [treeData, setTreeData] = useState([]); // 用户机构树
  const [tableData, setTableData] = useState([]); // 用户表
  const [currentSelect, setCurrentSelect] = useState([]);
  const [userVisible, setUserVisible] = useState(false);
  const [addOrgVisible, setAddOrgVisible] = useState(false);
  const [editOrgVisible, setEditOrgVisible] = useState(false);
  const [deleteOrgVisible, setDeleteOrgVisible] = useState(false);
  const [passwordVisible, setPasswordVisible] = useState(false);
  const [editUserVisible, setEditUserVisible] = useState(false);
  const [userState, setUserState] = useState(0);
  const [freezeUserVisible, setFreezeUserVisible] = useState(false);
  const [deleteUserVisible, setDeleteUserVisible] = useState(false);
  const [orgTitle, setOrgTitle] = useState('机构');
  const [orgID, setOrgID] = useState();
  const [userID, setUserID] = useState();
  const [addUserForm] = Form.useForm();
  const [addOrgForm] = Form.useForm();
  const [editOrgForm] = Form.useForm();
  const [editUserForm] = Form.useForm();
  const [passwordForm] = Form.useForm();
  let freezeText = '即将冻结该用户，是否确认冻结？';
  if (userState) {
    freezeText = '即将激活该用户，是否确认激活？';
  }
  // 渲染机构目录树
  const mapTree = org => {
    const haveChildren = Array.isArray(org.children) && org.children.length > 0;
    return {
      title: (
        <>
          <span>{org.text}</span>
          <div className={styles.iconWraper1}>
            <Tooltip title="添加用户">
              <FileAddTwoTone onClick={e => addUser(e, org.text, org.id)} />
            </Tooltip>
            <Tooltip title="添加下级机构">
              <FolderAddTwoTone onClick={e => addSubOrg(e, org.text, org.id)} />
            </Tooltip>
            <Tooltip title="编辑当前机构">
              <EditTwoTone onClick={e => editOrg(e, org.text, org.id)} />
            </Tooltip>
            <Tooltip title="删除当前机构">
              <DeleteTwoTone onClick={e => deleteOrg(e, org.text, org.id)} />
            </Tooltip>
          </div>
        </>
      ),
      key: org.id,
      // icon: <CaretDownOutlined />,
      // 判断它是否存在子集，若果存在就进行再次进行遍历操作，知道不存在子集便对其他的元素进行操作
      children: haveChildren ? org.children.map(i => mapTree(i)) : [],
    };
  };
  // 用户表列名
  const columns = [
    {
      title: '登录名',
      dataIndex: 'loginName',
      key: 'loginName',
    },
    {
      title: '用户姓名',
      dataIndex: 'userName',
      key: 'userName',
    },
    {
      title: '手机号码',
      dataIndex: 'phone',
      key: 'phone',
    },
    {
      title: '钉钉账户',
      dataIndex: 'ddid',
      key: 'ddid',
    },
    {
      title: '微信账户',
      dataIndex: 'wxid',
      key: 'wxid',
    },
    {
      title: '操作',
      key: 'action',
      align: 'center',
      render: (text, record) => (
        <Space size="middle">
          <Tooltip title="关联角色">
            <ShareAltOutlined
              onClick={() => relateRole(record)}
              style={{ fontSize: '16px', color: '#1890FF' }}
            />
          </Tooltip>
          <Tooltip title="更改机构">
            <ApartmentOutlined
              onClick={() => changeOrg(record)}
              style={{ fontSize: '16px', color: '#1890FF' }}
            />
          </Tooltip>
          <Tooltip title="修改密码">
            <KeyOutlined
              onClick={() => changePassword(record)}
              style={{ fontSize: '16px', color: '#1890FF' }}
            />
          </Tooltip>
          <Tooltip title="编辑用户">
            <EditTwoTone
              onClick={() => editUser(record)}
              style={{ fontSize: '16px' }}
            />
          </Tooltip>
          <Tooltip title="冻结用户">
            <PlayCircleOutlined
              onClick={() => freezeUser(record)}
              style={{ fontSize: '16px', color: '#1890FF' }}
            />
          </Tooltip>
          <Tooltip title="删除用户">
            <DeleteTwoTone
              onClick={() => deleteUser(record)}
              style={{ fontSize: '16px' }}
            />
          </Tooltip>
          {/* <a onClick={() => relateRole(record)}>关联角色 {record.name}</a>
          <a onClick={() => changeOrg(record)}>更改机构</a>
          <a onClick={() => changePassword(record)}>修改密码</a>
          <a onClick={() => editUser(record)}>编辑</a>
          <a
            onClick={() => freezeUser(record)}
            className={classnames({
              [styles.freeze]: userState,
              [styles.freezeGray]: !userState,
            })}
          >
            冻结
          </a>
          <a onClick={() => deleteUser(record)}>删除</a> */}
        </Space>
      ),
    },
  ];

  // 获取用户机构列表
  useEffect(() => {
    getUserTree(-1, -1);
  }, []);

  const getUserTree = (selectOU, node) => {
    setTreeLoading(true);
    get(`/Cityinterface/rest/services/OMS.svc/U_GetOUTree`, {
      _version: 9999,
      _dc: time.getTime(),
      selectOU,
      node,
    })
      .then(res => {
        setTreeLoading(false);
        if (res.length > 0) {
          const orgTree = res.map(org => mapTree(org));
          setTreeData(orgTree);
          onSelect([res[0].id]);
        }
      })
      .catch(err => {
        setTreeLoading(false);
        message.error(err);
      });
  };

  // 获取当前机构下所有用户
  const onSelect = props => {
    setTableLoading(true);
    if (!props[0]) {
      setCurrentSelect(currentSelect);
    } else {
      setCurrentSelect(props[0]);
    }
    get(`/Cityinterface/rest/services/OMS.svc/U_GetOneOUUserListNew`, {
      _version: 9999,
      _dc: time.getTime(),
      OUID: props[0] || currentSelect,
    })
      .then(res => {
        setTableLoading(false);
        const table = res.root.map((item, index) => {
          item.key = index;
          item.ddid = item.ddid ? '已绑定' : '未绑定';
          item.wxid = item.wxid ? '已绑定' : '未绑定';
          return item;
        });
        setTableData(table);
      })
      .catch(err => {
        setTableLoading(false);
        message.error(err);
      });
  };

  // 左侧目录树相关操作
  const addUser = (e, title, id) => {
    // e.stopPropagation();
    setUserVisible(true);
    setOrgTitle(`在${title}下添加用户`);
    setOrgID(id);
  };
  const addSubOrg = (e, title, id) => {
    // e.stopPropagation();
    setAddOrgVisible(true);
    setOrgTitle(`在${title}下添加机构`);
    setOrgID(id);
  };
  const editOrg = (e, title, id) => {
    // e.stopPropagation();
    setEditOrgVisible(true);
    editOrgForm.setFieldsValue({
      OUName: title,
    });
    setOrgID(id);
  };
  const deleteOrg = (e, title, id) => {
    // e.stopPropagation();
    setDeleteOrgVisible(true);
    setOrgTitle(`在${title}下添加用户`);
    setOrgID(id);
  };

  // 右侧表格相关操作
  const relateRole = record => {
    // setDeleteUserVisible(true);
    setUserID(record.userID);
  };
  const changeOrg = record => {
    // setDeleteUserVisible(true);
    setUserID(record.userID);
  };
  const changePassword = record => {
    setPasswordVisible(true);
    passwordForm.setFieldsValue({
      oldpassword: record.password,
    });
    setUserID(record.userID);
  };
  const editUser = record => {
    // console.log(record);
    setEditUserVisible(true);
    editUserForm.setFieldsValue({
      loginName: record.loginName,
      userName: record.userName,
      phone: record.phone || '',
      email: record.email || '',
    });
    setUserID(record.userID);
  };
  const freezeUser = record => {
    setFreezeUserVisible(true);
    setUserID(record.userID);
  };
  const deleteUser = record => {
    setDeleteUserVisible(true);
    setUserID(record.userID);
  };
  // 提交-添加用户
  const submitAddUser = () => {
    get(`/Cityinterface/rest/services/OMS.svc/U_AddUser`, {
      _version: 9999,
      _dc: time.getTime(),
      OUID: orgID,
      loginName: addUserForm.getFieldValue('loginName'),
      userName: addUserForm.getFieldValue('userName'),
      password: addUserForm.getFieldValue('password'),
      phone: addUserForm.getFieldValue('phone') || '',
      email: addUserForm.getFieldValue('email') || '',
    })
      .then(res => {
        if (res.success) {
          setUserVisible(false);
          notification.success({
            message: '提交成功',
          });
          // 重新获取用户表
          onSelect([orgID]);
        } else {
          notification.error({
            message: '提交失败',
            description: res.message,
          });
        }
      })
      .catch(err => {
        message.error(err);
      });
  };
  // 提交-添加下级机构
  const submitAddOrg = () => {
    get(`/Cityinterface/rest/services/OMS.svc/U_AddOU`, {
      _version: 9999,
      _dc: time.getTime(),
      pid: orgID,
      OUName: addOrgForm.getFieldValue('OUName'),
      description: addOrgForm.getFieldValue('desrciption') || '',
      comment: '',
    })
      .then(res => {
        if (res.success) {
          setAddOrgVisible(false);
          notification.success({
            message: '提交成功',
          });
          // 重新获取机构树与用户表
          getUserTree(-1, -1);
          // onSelect([res.OUID]);
        } else {
          notification.error({
            message: '提交失败',
            description: res.message,
          });
          // Modal.error({
          //   title: '添加失败',
          //   content: res.message,
          // });
        }
      })
      .catch(err => {
        message.error(err);
      });
  };
  // 提交-编辑当前机构
  const submitEditOrg = () => {
    get(`/Cityinterface/rest/services/OMS.svc/U_EditOneOUInfo`, {
      _version: 9999,
      _dc: time.getTime(),
      OUID: orgID,
      OUName: editOrgForm.getFieldValue('OUName'),
      description: editOrgForm.getFieldValue('desrciption') || '',
      comment: '',
    })
      .then(res => {
        if (res.success) {
          setEditOrgVisible(false);
          notification.success({
            message: '提交成功',
          });
          getUserTree(-1, -1);
          // 重新获取用户表
          // onSelect([orgID]);
        } else {
          notification.error({
            message: '提交失败',
            description: res.message,
          });
        }
      })
      .catch(err => {
        message.error(err);
      });
  };
  const submitDeleteOrg = () => {
    get(`/Cityinterface/rest/services/OMS.svc/U_DeleteOU`, {
      _version: 9999,
      _dc: time.getTime(),
      OUID: orgID,
    })
      .then(res => {
        if (res.success) {
          setDeleteOrgVisible(false);
          notification.success({
            message: '提交成功',
          });
          // 重新获取用户表
          getUserTree(-1, -1);
          // onSelect([orgID]);
        } else {
          notification.error({
            message: '提交失败',
            description: res.message,
          });
        }
      })
      .catch(err => {
        setTableLoading(false);
        message.error(err);
      });
  };

  const submitChangePassword = () => {
    get(`/Cityinterface/rest/services/OMS.svc/U_UpdatePassword`, {
      _version: 9999,
      _dc: time.getTime(),
      userID,
      oldpassword: passwordForm.getFieldValue('password'),
      newPassword: passwordForm.getFieldValue('newPassword'),
      passwordConfirm: passwordForm.getFieldValue('passwordConfirm'),
    })
      .then(res => {
        if (res.success) {
          setPasswordVisible(false);
          // 重新获取用户表
          onSelect([orgID]);
          notification.success({
            message: '提交成功',
          });
        } else {
          notification.error({
            message: '提交失败',
            description: res.message,
          });
        }
      })
      .catch(err => {
        message.error(err);
      });
  };
  const submitEditUser = () => {
    get(`/Cityinterface/rest/services/OMS.svc/U_EditUser`, {
      _version: 9999,
      _dc: time.getTime(),
      userID,
      loginName: editUserForm.getFieldValue('loginName'),
      userName: editUserForm.getFieldValue('userName'),
      phone: editUserForm.getFieldValue('phone') || '',
      email: editUserForm.getFieldValue('email') || '',
    })
      .then(res => {
        if (res.success) {
          setEditUserVisible(false);
          // 重新获取用户表
          onSelect([orgID]);
          notification.success({
            message: '提交成功',
          });
        } else {
          notification.error({
            message: '提交失败',
            description: res.message,
          });
        }
      })
      .catch(err => {
        setTableLoading(false);
        message.error(err);
      });
  };
  const submitFreezeUser = () => {
    get(`/Cityinterface/rest/services/OMS.svc/U_UserStateOU`, {
      _version: 9999,
      _dc: time.getTime(),
      OUID: userID,
      state: +!userState,
    })
      .then(res => {
        if (res.success) {
          setFreezeUserVisible(false);
          // 重新获取用户表
          onSelect([orgID]);
          notification.success({
            message: '提交成功',
          });
          setUserState(+!userState);
        } else {
          notification.error({
            message: '提交失败',
            description: res.message,
          });
        }
      })
      .catch(err => {
        setTableLoading(false);
        message.error(err);
      });
  };
  const submitDeleteUser = () => {
    get(`/Cityinterface/rest/services/OMS.svc/U_DeleteUser`, {
      _version: 9999,
      _dc: time.getTime(),
      userID,
    })
      .then(res => {
        if (res.success) {
          setDeleteUserVisible(false);
          notification.success({
            message: '提交成功',
          });
          // 重新获取用户表
          onSelect([orgID]);
        } else {
          notification.error({
            message: '提交失败',
            description: res.message,
          });
        }
      })
      .catch(err => {
        setTableLoading(false);
        message.error(err);
      });
  };

  return (
    <PageContainer className={styles.userManageContainer}>
      <div className={styles.contentContainer}>
        <div className={styles.orgContainer}>
          <span style={{ padding: '8px', fontSize: '16px' }}>
            机构列表
            <Tooltip title="添加下级机构">
              <FolderAddTwoTone
                style={{ fontSize: '16px', float: 'right' }}
                onClick={e => addSubOrg(e, '根目录', '-1')}
              />
            </Tooltip>
          </span>
          <Tree
            showLine="true"
            showIcon="true"
            selectedKeys={[currentSelect]}
            loading={treeLoading}
            onSelect={onSelect}
            treeData={treeData}
          />
        </div>
        <div className={styles.userContainer}>
          <Table
            columns={columns}
            dataSource={tableData}
            loading={tableLoading}
          />
        </div>
        {/* 添加用户 */}
        <Modal
          title={orgTitle}
          visible={userVisible}
          onOk={submitAddUser}
          onCancel={() => setUserVisible(false)}
          okText="确认"
          cancelText="取消"
        >
          <Form form={addUserForm}>
            <Form.Item
              name="loginName"
              label="登录名称"
              rules={[
                {
                  required: true,
                },
              ]}
            >
              <Input placeholder="请输入登录名称" />
            </Form.Item>
            <Form.Item
              name="password"
              label="账户密码"
              rules={[
                {
                  required: true,
                },
              ]}
            >
              <Input placeholder="请输入账户密码" />
            </Form.Item>
            <Form.Item
              name="userName"
              label="用户名称"
              rules={[
                {
                  required: true,
                },
              ]}
            >
              <Input placeholder="请输入用户姓名" />
            </Form.Item>
            <Form.Item name="phone" label="手 机 号 码">
              <Input placeholder="请输入手机号码" />
            </Form.Item>
            <Form.Item name="email" label="电 子 邮 箱">
              <Input placeholder="请输入电子邮箱" />
            </Form.Item>
          </Form>
        </Modal>
        {/* 添加下级机构 */}
        <Modal
          title={orgTitle}
          visible={addOrgVisible}
          onOk={submitAddOrg}
          onCancel={() => setAddOrgVisible(false)}
          okText="确认"
          cancelText="取消"
        >
          <Form form={addOrgForm}>
            <Form.Item
              name="OUName"
              label="机构名称"
              rules={[
                {
                  required: true,
                },
              ]}
            >
              <Input placeholder="请输入机构名称" />
            </Form.Item>
            <Form.Item name="description" label="--描---述--">
              <Input placeholder="请输入相关描述" />
            </Form.Item>
          </Form>
        </Modal>
        {/* 编辑机构 */}
        <Modal
          title={orgTitle}
          visible={editOrgVisible}
          onOk={submitEditOrg}
          onCancel={() => setEditOrgVisible(false)}
          okText="确认"
          cancelText="取消"
        >
          <Form form={editOrgForm}>
            <Form.Item
              name="OUName"
              label="机构名称"
              rules={[
                {
                  required: true,
                },
              ]}
            >
              <Input placeholder="请输入机构名称" />
            </Form.Item>
            <Form.Item name="description" label="--描---述--">
              <Input placeholder="请输入相关描述" />
            </Form.Item>
          </Form>
        </Modal>
        <Modal
          title="确认删除机构"
          visible={deleteOrgVisible}
          onOk={submitDeleteOrg}
          onCancel={() => setDeleteOrgVisible(false)}
          okText="确认"
          cancelText="取消"
        >
          <p>即将删除该机构，是否确认删除？</p>
        </Modal>
        {/* 修改密码 */}
        <Modal
          title="编辑用户"
          visible={passwordVisible}
          onOk={submitChangePassword}
          onCancel={() => setPasswordVisible(false)}
          okText="确认"
          cancelText="取消"
        >
          <Form form={passwordForm}>
            <Form.Item name="oldpassword" label="原 始 密 码">
              <Input disabled />
            </Form.Item>
            <Form.Item
              name="newPassword"
              label="新- 密-码"
              rules={[
                {
                  required: true,
                },
              ]}
            >
              <Input placeholder="请输入新密码" />
            </Form.Item>
            <Form.Item
              name="passwordConfirm"
              label="确认密码"
              rules={[
                {
                  required: true,
                },
              ]}
            >
              <Input placeholder="再次确认新密码" />
            </Form.Item>
          </Form>
        </Modal>
        {/* 编辑用户 */}
        <Modal
          title="编辑用户"
          visible={editUserVisible}
          onOk={submitEditUser}
          onCancel={() => setEditUserVisible(false)}
          okText="确认"
          cancelText="取消"
        >
          <Form form={editUserForm}>
            <Form.Item
              name="loginName"
              label="登录名称"
              rules={[
                {
                  required: true,
                },
              ]}
            >
              <Input placeholder="请输入登录名称" />
            </Form.Item>
            <Form.Item
              name="userName"
              label="用户姓名"
              rules={[
                {
                  required: true,
                },
              ]}
            >
              <Input placeholder="请输入用户姓名" />
            </Form.Item>
            <Form.Item name="phone" label="手 机 号 码">
              <Input placeholder="请输入手机号码" />
            </Form.Item>
            <Form.Item name="email" label="电 子 邮 箱">
              <Input placeholder="请输入电子邮箱" />
            </Form.Item>
          </Form>
        </Modal>
        <Modal
          title="请确认冻结用户"
          visible={freezeUserVisible}
          onOk={submitFreezeUser}
          onCancel={() => setFreezeUserVisible(false)}
          okText="确认"
          cancelText="取消"
        >
          <p>{freezeText}</p>
        </Modal>
        <Modal
          title="确认删除用户"
          visible={deleteUserVisible}
          onOk={submitDeleteUser}
          onCancel={() => setDeleteUserVisible(false)}
          okText="确认"
          cancelText="取消"
        >
          <p>即将关联删除 主站点 用户关系，是否确认删除？</p>
        </Modal>
      </div>
    </PageContainer>
  );
};

export default UserManage;
